@@ -13,6 +13,13 @@ from xml.sax import saxlib, saxexts
 # It's also used as a SAX handler, which may be a good idea but may
 # also be a stupid hack.
 
+def version_independent_cmp(a,b):
+    ta = type(a)
+    tb = type(b)
+    if ta is not tb:
+        return cmp(ta.__name__, tb.__name__)
+    return cmp(a,b)
+
 class Marshaller(saxlib.HandlerBase):
     # XML version and DOCTYPE declaration
     PROLOGUE = '<?xml version="1.0"?>'
@@ -114,8 +121,12 @@ class Marshaller(saxlib.HandlerBase):
         L.append( s )
         L.append( '</' + name + '>')
         return L
-    # 2.2a1 work-around: the string type was renamed to str
-    m_str = m_string
+
+    # Since Python 2.2, the string type has a name of 'str'
+    # To avoid having to rewrite all classes that implement m_string
+    # we delegate m_str to m_string.
+    def m_str(self, value, dict):
+        return self.m_string(value, dict)
     
     def m_int(self, value, dict):
         name = self.tag_int ; L = []
@@ -162,16 +173,24 @@ class Marshaller(saxlib.HandlerBase):
         items = value.items()
         # Sort the items to allow reproducable results across Python
         # versions
-        items.sort()
+        items.sort(version_independent_cmp)
         for key, v in items:
             L = L + self._marshal(key, dict)
             L = L + self._marshal(v, dict)
         L.append( '</' + name + '>')
         return L
 
+    # Python 2.2 renames dictionary to dict.
+    def m_dict(self, value, dict):
+        return self.m_dictionary(value, dict)
+
     def m_None(self, value, dict):
         return ['<' + self.tag_none + '/>']
 
+    # Python 2.2 renamed the type of None to NoneTye
+    def m_NoneType(self, value, dict):
+        return self.m_None(value, dict)
+
     def m_complex(self, value, dict):
         name = self.tag_complex ; L = []
         return [ '<' + name + '>' + str(value.real) + ' ' + str(value.imag)
