@@ -4,7 +4,7 @@ one, so this module is the only one one needs to import. For validating
 parsing, import xmlval instead.
 """
 
-# $Id: xmlproc.py,v 1.13 2001/01/14 10:42:24 loewis Exp $
+# $Id: xmlproc.py,v 1.21 2001/08/28 06:55:34 larsga Exp $
    
 import re,string,sys,urllib,urlparse
 
@@ -17,7 +17,7 @@ from xmlapp import *
 from xmldtd import *
 
 version="0.70"
-revision="$Revision: 1.13 $"
+revision="$Revision: 1.21 $"
         
 # ==============================
 # A full well-formedness parser
@@ -86,10 +86,7 @@ class XMLProcessor(XMLCommonParser):
 		self.prepos=self.pos
 
 		if self.data[self.pos]=="<":
-                    try:
-                        t=self.data[self.pos+1] # Optimization
-                    except IndexError,e:            
-                        raise OutOfDataException()
+                    t=self.data[self.pos+1] # Optimization
                     if t=="/":
                         self.parse_end_tag()
                     elif t!="!" and t!="?":
@@ -113,7 +110,20 @@ class XMLProcessor(XMLCommonParser):
                         self.parse_ent_ref()
                 else:
                     self.parse_data()
-	except OutOfDataException,e:
+
+        except IndexError, e:            
+            # Means self.pos was outside the buffer when we did a raw
+            # compare.  This is both a little ugly and fragile to
+            # changes, but this loop is rather time-critical, so we do
+            # raw compares anyway.
+            # Should try to lose this since it gets very hard to find
+            # problems if the user throws an IndexError...
+            
+	    if self.final:
+		raise OutOfDataException()
+	    else:
+		self.pos=self.prepos  # Didn't complete the construct        
+	except OutOfDataException, e:
 	    if self.final:
 		raise e
 	    else:
@@ -209,7 +219,7 @@ class XMLProcessor(XMLCommonParser):
 	        
         while 1:
             piece=self.find_reg(reg_attval_stop)
-            val=val+string_translate(piece,ws_trans)
+            val=val+ws_trans(piece)
 
 	    if self.now_at(delim):
                 break
@@ -241,7 +251,7 @@ class XMLProcessor(XMLCommonParser):
                         self.pop_entity()
                     else:
                         self.report_error(3020)
-                except KeyError,e:
+                except KeyError:
                     self.report_error(3021,name) ## FIXME: Check standalone dcl
 
                 del self.open_ents[-1]
@@ -268,13 +278,13 @@ class XMLProcessor(XMLCommonParser):
         while 1:
             try:
                 piece=self.find_reg(reg_stop)
-            except OutOfDataException,e:
+            except OutOfDataException:
                 # Only character data left
-                val=val+string_translate(self.data[self.pos:],ws_trans)
+                val=val+ws_trans(self.data[self.pos:])
                 self.pos=self.datasize
                 break
-            
-            val=val+string_translate(piece,ws_trans)
+
+            val=val+ws_trans(piece)
 
 	    if self.now_at("&#"):
                 val=val+self._read_char_ref()		
@@ -303,7 +313,7 @@ class XMLProcessor(XMLCommonParser):
                         self.pop_entity()
                     else:
                         self.report_error(3020)
-                except KeyError,e:
+                except KeyError:
                     self.report_error(3021,name)	       
 
                 del self.open_ents[-1]
@@ -340,7 +350,7 @@ class XMLProcessor(XMLCommonParser):
                 else:
                     self.stack.append(elem) # Put it back
 
-	except IndexError,e:
+	except IndexError:
 	    self.report_error(3024,name)
 
         self.app.handle_end_tag(name)
@@ -383,7 +393,7 @@ class XMLProcessor(XMLCommonParser):
 	else:
             try:
                 digs=int(self.get_match(reg_digits))
-            except ValueError,e:
+            except ValueError:
                 self.report_error(3027)
                 digs=None
 
@@ -393,13 +403,16 @@ class XMLProcessor(XMLCommonParser):
 	if not (digs==9 or digs==10 or digs==13 or \
 		(digs>=32 and digs<=255)):
 	    if digs>255:
-		self.report_error(1005,digs)
+                if using_unicode and digs<65536:
+                    self.app.handle_data(xml_chr(digs),0,1)
+                else:
+                    self.report_error(1005,digs)
 	    else:
 		self.report_error(3018,digs)
 	else:
 	    if self.stack==[]:
 		self.report_error(3028)
-	    self.app.handle_data(chr(digs),0,1)
+	    self.app.handle_data(xml_chr(digs),0,1)
 
     def parse_cdata(self):
 	"Parses a CDATA marked section from after the '<![CDATA['."
@@ -416,7 +429,7 @@ class XMLProcessor(XMLCommonParser):
 
         try:
             ent=self.ent.resolve_ge(name)
-	except KeyError,e:
+	except KeyError:
 	    self.report_error(3021,name)
             return
 
@@ -442,20 +455,17 @@ class XMLProcessor(XMLCommonParser):
 	    self.flush()
 	    self.pop_entity()
 	else:
-	    if ent.notation!="":
+	    if ent.notation != None:
 		self.report_error(3031)
-
-            tmp=self.seen_xmldecl
-            self.seen_xmldecl=0 # Avoid complaints
-            self.seen_root=0    # Haven't seen root in the new entity yet
-            self.open_entity(self.pubres.resolve_entity_pubid(ent.get_pubid(),
-                                                              ent.get_sysid()),
-                             name)
-            self.seen_root=1 # Entity references only allowed inside elements
-            self.seen_xmldecl=tmp
+            else:
+                self.seen_root = 0    # Haven't seen root in the new entity yet
+                self.open_entity(self.pubres.resolve_entity_pubid(ent.get_pubid(),
+                                                                  ent.get_sysid()),
+                                 name)
+                self.seen_root = 1
 
         # Did any elements cross the entity boundary?
-        if stack_size!=len(self.stack):
+        if stack_size != len(self.stack):
             self.report_error(3042)
             
 	del self.open_ents[-1]
@@ -469,10 +479,10 @@ class XMLProcessor(XMLCommonParser):
 	    self.report_error(3033)
 	
 	self.skip_ws(1)
-        rootname=self._get_name()
+        rootname = self._get_name()
 	self.skip_ws(1)
 
-        (pub_id,sys_id)=self.parse_external_id()
+        (pub_id, sys_id) = self.parse_external_id()
 	self.skip_ws()
 
         self.app.handle_doctype(rootname, pub_id, sys_id)
@@ -481,14 +491,18 @@ class XMLProcessor(XMLCommonParser):
 	if self.now_at("["):
 	    self.parse_internal_dtd()    
 	elif not self.now_at(">"):
-            self.report_error(3005,">")
+            self.report_error(3005, ">")
 
         # External subset must be parsed _after_ the internal one
-	if pub_id!=None or sys_id!=None: # Was there an external id at all?
+	if pub_id != None or sys_id != None: # Was there an external id at all?
+            if not self.get_current_sysid() and \
+               urlparse.urlparse(sys_id)[0] == "":
+                self.report_error(2024, sys_id)
+            
             if self.read_external_subset:
                 try:
                     sys_id = self.pubres.resolve_doctype_pubid(pub_id, sys_id)
-                    p=self._setup_dtd_parser(0)
+                    p = self._setup_dtd_parser(0)
                     p.dtd_start_called = 1
                     p.parse_resource(join_sysids(self.get_current_sysid(),
                                                  sys_id))
@@ -541,8 +555,8 @@ class XMLProcessor(XMLCommonParser):
 		
 		p.set_sysid(self.get_current_sysid())
                 p.final=1
-		p.feed(int_dtd)
-	    except OutOfDataException,e:
+		p.feed(int_dtd, decoded = 1)
+	    except OutOfDataException:
 		self.report_error(3034)
 	finally:
             p.deref()
